//********************************************************************
// File EbApiFile.h        Low-level file & modules handling functions
//********************************************************************
// Copyright @2013 by Stan Sykora and Extra Byte
#ifndef _EbApiFile_h_
#define _EbApiFile_h_

#include "EbApiBase.h"
#include "EbBasicLib.h"
#include "EbInt64.h"

// Note: All File and Directory names, as well as all path strings
// are now in wide characters (wchar_t, LPWSTR), while
// file contents are assumed to be ASCI

#define MxPATH 256

// File handling flags
//--------------------------------------------------------------------
enum fFLAGS {
      fDefaults=		0,       // 0's correspond to defaults
	fRead=			0,		// Access
        fWrite=			0x1,            // (Higher value wins)
      fAccM=			0x1,		// Access mask
	fMayExist=		0,		// Existence
        fMustExist=		0x2,            // (Higher value wins)
        fNew=			0x4,
      fExiM=			0x6,		// Existence mask
	fRewind=		0,		// Initialization
        fAppend=		0x08,           // (Higher value wins)
        fTruncate=		0x10,
      fIniM=			0x18,		// Initialization mask
        fShareDefault=		0,		// Sharing
        fShareRead=		0x020,		// (Higher value wins)
        fShareWrite=		0x040,
	fShare=			0x080,
        fShareNot=		0x100,
      fShrM=			0x1E0,		// Sharing mask
        fNormal=		0,		// Attributes
        fReadOnly=		0x00200,
        fHidden=		0x00400,
        fSystem=		0x00800,
	fArchive=		0x01000,
        fDirectory=		0x02000,
        fCompressed=		0x04000,
        fTemporary=		0x08000,
        fOffline=		0x10000,
      fAtrM=			0x1FE00,	// Attributes mask
        fNoBeep=		0x20000,	// File Open Dialogs
        fOverwritePrompt=	0x40000,
        fNoDereference=		0x80000,
        fUseWinFlags=	       0x100000
};

// File handling errors
//--------------------------------------------------------------------
  #define FERR_FileSystemCrash  0x80000000
  #define FERR_BadDiscriminator 0x00000400
  #define FERR_DiskAbsent       0x00000200
  #define FERR_DiskNotDos       0x00000100
  #define FERR_DiskNotReady     0x00000080
  #define FERR_WrongDisk        0x00000040
  #define FERR_DiskFull         0x00000020
  #define FERR_WriteProtected   0x00000010
  #define FERR_PathAbsent       0x00000008
  #define FERR_FileAbsent       0x00000004
  #define FERR_FileSharingErr   0x00000002
  #define FERR_FileNoAccess     0x00000001

// EbFILEINFO structures
//--------------------------------------------------------------------

typedef struct _EbFILEINFO {
  DWORD  Flags;
  DWORD  Attributes;
  cInt64 Size;
  cInt64 CompressedSize;
  cInt64 CreationFT;
  cInt64 LastAccessFT;
  cInt64 LastWriteFT;
  DWORD  BinaryType;
  DWORD  VersionInfoSize;
  union {
    DWORD ProductVersionMS;
    struct {
      WORD ProductMinorVersion;
      WORD ProductMajorVersion;
    };
  };
  union {
    DWORD ProductVersionLS;
    struct {
      WORD ProductBuild;
      WORD ProductRelease;
    };
  };
  union {
    DWORD FileVersionMS;
    struct {
      WORD FileMinorVersion;
      WORD FileMajorVersion;
    };
  };
  union {
    DWORD FileVersionLS;
    struct {
      WORD FileBuild;
      WORD FileRelease;
    };
  };
  DWORD  FileInfoFlags;
  DWORD  FileType;
  DWORD  FileSubType;
  DWORD  FileOS;
  DWORD  LoFileVersionFT;
  DWORD  HiFileVersionFT;
  DWORD  TranslationCount;
  union {                                 // Coming from SHGetFileInfo
    DWORD  ExeType;
    struct {
      char ExeChar[2];				       // PE,NE,MZ,...
      WORD ExeVersion;                              // Windows version
    };
  };
  DWORD	 ShellAttributes;
  DWORD  ShellIconIndex;
  WORD*  TranslationTable;	           // Points inside InfoBuffer
  LPSTR	 ShellName;                            // Dynamic allocation!!
  LPSTR	 ShellType;                            // Dynamic allocation!!
  LPSTR	 ShellIconLocation;                    // Dynamic allocation!!
  LPSTR  InfoBuffer;                           // Dynamic allocation!!
} EbFILEINFO;

typedef struct _EbFILEINFOW {
  DWORD  Flags;
  DWORD  Attributes;
  cInt64 Size;
  cInt64 CompressedSize;
  cInt64 CreationFT;
  cInt64 LastAccessFT;
  cInt64 LastWriteFT;
  DWORD  BinaryType;
  DWORD  VersionInfoSize;
  union {
    DWORD ProductVersionMS;
    struct {
      WORD ProductMinorVersion;
      WORD ProductMajorVersion;
    };
  };
  union {
    DWORD ProductVersionLS;
    struct {
      WORD ProductBuild;
      WORD ProductRelease;
    };
  };
  union {
    DWORD FileVersionMS;
    struct {
      WORD FileMinorVersion;
      WORD FileMajorVersion;
    };
  };
  union {
    DWORD FileVersionLS;
    struct {
      WORD FileBuild;
      WORD FileRelease;
    };
  };
  DWORD  FileInfoFlags;
  DWORD  FileType;
  DWORD  FileSubType;
  DWORD  FileOS;
  DWORD  LoFileVersionFT;
  DWORD  HiFileVersionFT;
  DWORD  TranslationCount;
  union {                                 // Coming from SHGetFileInfo
    DWORD  ExeType;
    struct {
      char ExeChar[2];				       // PE,NE,MZ,...
      WORD ExeVersion;                              // Windows version
    };
  };
  DWORD	 ShellAttributes;
  DWORD  ShellIconIndex;
  WORD*  TranslationTable;	           // Points inside InfoBuffer
  LPWSTR ShellName;                            // Dynamic allocation!!
  LPWSTR ShellType;                            // Dynamic allocation!!
  LPWSTR ShellIconLocation;                    // Dynamic allocation!!
  LPWSTR InfoBuffer;                           // Dynamic allocation!!
} EbFILEINFOW;

// EbFILEINFO flags (high word reserved for FIF flags of cEbFileInfo
//--------------------------------------------------------------------
#define FIF_Exists		0x00000001
#define FIF_GotAttributes	0x00000002
#define FIF_BinaryType		0x00000004
#define FIF_GotSize		0x00000008
#define FIF_GotCompressedSize	0x00000010
#define FIF_GotTimes		0x00000020
#define FIF_AlternateName	0x00000040
#define FIF_GotFileVersion	0x00000080
#define FIF_GotProductVersion	0x00000100
#define FIF_GotFileFlags	0x00000200
#define FIF_GotFileType		0x00000400
#define FIF_GotFileOS		0x00000800
#define FIF_GotFileVersionTime	0x00001000
#define FIF_GotTranslationCount	0x00002000
#define FIF_GotInfoBuffer	0x00004000
#define FIF_GotTranslationTable	0x00008000
#define FIF_Executable		0x00010000
#define FIF_ShellName		0x00020000
#define FIF_ShellType		0x00040000
#define FIF_ShellAttributes	0x00080000
#define FIF_ShellIcon     	0x00100000
#define FIF_ShellIconLocation  	0x00200000

//********************************************************************
// Globals initialized upon loading
//********************************************************************
extern DWORD _LastOsFileError;       // Handle this through functions!
                              // Current application (MyApp) examples:
extern cStrAW _FullExePath;                 // D:MyDir/Hobby/MyApp.exe
extern cStrAW _CommandLine;                     // InputData.txt /c -w
extern cStrAW _ExeName;                                   // MyApp.exe
extern cStrAW _RootName;                                      // MyApp
extern cStrAW _ExeDir;                               // D:MyDir/Hobby/
extern cStrAW _CurrentDir;                           // D:MyDir/Hobby/
extern cStrAW _WindowsDir;                               // C:/Windows
extern cStrAW _SystemDir;                         // C:/Windows/System
extern cStrAW _TempDir;                  // C:/Windows/Temp or similar

extern BOOL _stdcall (*_GetThreadTimesAPI_)
        (HANDLE,EbFILETIME*,EbFILETIME*,EbFILETIME*,EbFILETIME*);
extern BOOL _stdcall (*_GetProcessTimesAPI_)
        (HANDLE,EbFILETIME*,EbFILETIME*,EbFILETIME*,EbFILETIME*);
extern BOOL _stdcall (*_CancelIO_)(HANDLE);
//********************************************************************

// Initialization functions
// They should be run only once; afterwards there is no need to do so,
// -------------------------------------------------------------------
  LPWSTR  _GetCommandLine();
  LPSTR   _CopyOfWindowsDirectory();
  LPWSTR  _CopyOfWindowsDirectoryW();
  LPSTR   _CopyOfSystemDirectory();
  LPWSTR  _CopyOfSystemDirectoryW();
  LPSTR   _CopyOfTempPath();
  LPWSTR  _CopyOfTempPathW();

// Auxiliary and file-handling error functions
// -------------------------------------------------------------------

  RETOK   _KillFileHandle(HANDLE* ph);

  inline void   _ClearLastFileError(){_LastOsFileError = 0;}
  inline RETERR _GetLastFileError(){return _LastOsFileError;}
  RETERR  _RefreshLastFileError();
  DWORD   _FileErrorEbCode(DWORD ferr);

  void    _InitializeFileInfo(EbFILEINFO* fi);
  void    _EraseFileInfo(EbFILEINFO* fi);

// Disk functions
// -------------------------------------------------------------------

  DWORD   _GetLogicalDrivesCode();
  DWORD   _GetLogicalDrivesString(LPSTR bfr,LPCSTR trailer);

// Directory functions
//--------------------------------------------------------------------

  LPSTR   _CopyOfCurrentDirectory();
  LPWSTR  _CopyOfCurrentDirectoryW();

  RETOK   _CreateDirectory(LPCSTR path);
  RETOK   _CreateDirectoryW(LPCWSTR path);

// Path and FileName functions
//--------------------------------------------------------------------

  LPSTR   _CopyOfTempFileName
		(LPCSTR path=NULL,LPCSTR prefix="",DWORD unum=0);
  LPWSTR  _CopyOfTempFileNameW
		(LPCWSTR path=NULL,LPCWSTR prefix=L"",DWORD unum=0);

  LPSTR   _CopyOfSearchPath(LPCSTR path,LPCSTR fname,
  		LPCSTR ext=NULL,LPSTR* cleanname=NULL);
  LPWSTR  _CopyOfSearchPathW(LPCWSTR path,LPCWSTR fname,
  		LPCWSTR ext=NULL,LPWSTR* cleanname=NULL);

  LPSTR   _CopyOfFullPathName(LPCSTR fname,LPSTR* cleanname=NULL);
  LPWSTR  _CopyOfFullPathNameW(LPCWSTR fname,LPWSTR* cleanname=NULL);

  LPSTR   _NewFullFileNameCombination(LPCSTR path,LPCSTR cleanname);
  LPWSTR  _NewFullFileNameCombinationW(LPCWSTR path,LPCWSTR cleanname);

  LPSTR   _CopyOfPathFromFullFileName(LPCSTR fullname);
  LPWSTR  _CopyOfPathFromFullFileNameW(LPCWSTR fullname);

  LPSTR   _CopyOfCleanNameFromFullFileName(LPCSTR fullname);
  LPWSTR  _CopyOfCleanNameFromFullFileNameW(LPCWSTR fullname);

  LPSTR   _CopyOfRootNameFromFullFileName(LPCSTR fullname);
  LPWSTR  _CopyOfRootNameFromFullFileNameW(LPCWSTR fullname);

  LPSTR   _CopyOfExtensionFromFullFileName(LPCSTR fullname);
  LPWSTR  _CopyOfExtensionFromFullFileNameW(LPCWSTR fullname);

  LPCSTR  _CleanNameInFullFileName(LPCSTR fullname);
  LPCWSTR _CleanNameInFullFileNameW(LPCWSTR fullname);

  LPCSTR  _ExtensionInFullFileName(LPCSTR fullname);
  LPCWSTR _ExtensionInFullFileNameW(LPCWSTR fullname);

  LPSTR   _NewFileName(LPCSTR title,LPCSTR inidir,
                       LPCSTR defext,LPCSTR filter,DWORD flags,
                       HANDLE ownerwin=NULL,DWORD* retflags=NULL);
  LPWSTR  _NewFileNameW(LPCWSTR title,LPCWSTR inidir,
                       LPCWSTR defext,LPCWSTR filter,DWORD flags,
                       HANDLE ownerwin=NULL,DWORD* retflags=NULL);

// File-find functions
//--------------------------------------------------------------------

  HANDLE _FindFirstFile(LPCSTR wildname,LPSTR* pcopyname,
                        EbFILEINFO* fi = NULL);
  HANDLE _FindFirstFileW(LPCWSTR wildname,LPWSTR* pcopyname,
                        EbFILEINFOW* fi = NULL);

  RETOK  _FindNextFile(HANDLE ffh,LPSTR* pcopyname,
                        EbFILEINFO* fi = NULL);
  RETOK  _FindNextFileW(HANDLE ffh,LPWSTR* pcopyname,
                        EbFILEINFOW* fi = NULL);

  RETOK  _FindClose(HANDLE ffh,LPSTR* pcopyname);
  RETOK  _FindCloseW(HANDLE ffh,LPWSTR* pcopyname);

// Info & Operations on closed files
// -------------------------------------------------------------------

  RETOK   _isFilePresent(LPCSTR fullfilename);
  RETOK   _isFilePresentW(LPCWSTR fullfilename);

  RETOK   _isFilePresent(LPCSTR path,LPCSTR cleanname);
  RETOK   _isFilePresentW(LPCWSTR path,LPCWSTR cleanname);

  RETOK   _isFileInExeDir(LPCSTR cleanname);
  RETOK   _isFileInExeDirW(LPCWSTR cleanname);

  DWORD   _FileAttributes(LPCSTR fullfilename);
  DWORD   _FileAttributesW(LPCWSTR fullfilename);

  DWORD   _FileAttributes(LPCSTR path,LPCSTR cleanname);
  DWORD   _FileAttributesW(LPCWSTR path,LPCWSTR cleanname);

  RETOK   _aFileExists(LPCSTR wildname,LPSTR* pcopyname = NULL,
                       EbFILEINFO* fi = NULL);
  RETOK   _aFileExistsW(LPCWSTR wildname);

  RETOK   _aFileExists(LPCSTR path,LPCSTR cleanname);
  RETOK   _aFileExistsW(LPCSTR path,LPCSTR cleanname);

  RETOK   _FileBinaryType(LPCSTR fullfilename,DWORD* lptype);

  RETOK   _EraseFile(LPCSTR fullfilename);
  RETOK   _EraseFileW(LPCWSTR fullfilename);

  RETOK   _EraseFiles(LPCSTR wildname);
  RETOK   _EraseFilesW(LPCWSTR wildname);

  RETOK   _FileCopy
              (LPCSTR fullfrom,LPCSTR fullto,DWORD failifexists);
  RETOK   _FileCopyW
              (LPCWSTR fullfrom,LPCWSTR fullto,DWORD failifexists);
  RETOK   _FileCopy(LPCSTR pathfrom,LPCSTR cleanfrom,
  	       LPCSTR pathto,LPCSTR cleanto,DWORD failifexists);
  RETOK   _FileCopyW(LPCWSTR pathfrom,LPCWSTR cleanfrom,
  	       LPCWSTR pathto,LPCWSTR cleanto,DWORD failifexists);

  RETOK   _CopyFileFromExeDirToSystemDir(LPCSTR cleanname);
  RETOK   _CopyFileFromExeDirToSystemDirW(LPCWSTR cleanname);
  RETOK   _CopyFileFromExeDirToSystemDirIfAbsent(LPCSTR cleanname);
  RETOK   _CopyFileFromExeDirToSystemDirIfAbsentW(LPCWSTR cleanname);

  HANDLE  _FileOpen(LPCSTR fname,DWORD flags);
  HANDLE  _FileOpenW(LPCWSTR fname,DWORD flags);

  RETOK   _CopyFiles(LPCSTR wildfrom,LPCSTR pathto,
                     LONG(*IfExists)(LPCSTR name));

// Info & Operations on open files
// -------------------------------------------------------------------

  // This one is private; don't use it directly
  DWORD   _SetFilePointer(HANDLE hFile,LONG lDistanceToMove,
              LONG* lpDistanceToMoveHigh,LONG refloc);

  RETOK   _FlushBuffers(HANDLE fileorport);
  BOOL    _SetEndOfFile(HANDLE hFile);
  RETOK   _TruncateFile(HANDLE fh);

  RETOK   _FileSize(HANDLE fh,cInt64* sz);
  RETOK   _CompressedFileSize(LPCSTR filename,cInt64* sz);
  RETOK   _SetFilePointer(HANDLE fh,cInt64* newptr);
  RETOK   _MoveFilePointerBy(HANDLE fh,cInt64* delta,cInt64* newptr);
  RETOK   _MoveToEndOfFile(HANDLE fh,cInt64* newptr);

  // The next four are deprecated (useful only for file sizes <= 2Gb)
  RETOK   _FileSize(HANDLE fh,DWORD* sz);
  RETOK   _SetFilePointer(HANDLE fh,LONG* newptr);
  RETOK   _MoveFilePointerBy(HANDLE fh,LONG* delta,LONG* newptr);
  RETOK   _MoveToEndOfFile(HANDLE fh,LONG* newptr);

  LONG    _ReadFromFile(HANDLE fh,void* buffer,DWORD bytes);
  RETOK   _WriteToFile (HANDLE fh,const void* buffer,DWORD bytes);

  RETOK   _GetFileTimes(HANDLE fh,EbFILETIME* creation,
		EbFILETIME* lastaccess,EbFILETIME* lastwrite);
  RETOK   _SetFileTimes(HANDLE fh,EbFILETIME* creation,
		EbFILETIME* lastaccess,EbFILETIME* lastwrite);

//********************************************************************
// Modules handling functions
//********************************************************************

  HANDLE _LoadDll(LPCSTR dllname);
  HANDLE _LoadDllW(LPCWSTR dllname);
  RETOK  _FreeDll(HANDLE *handle);
  void*  _ProcAddress(HANDLE hmodule,LPCSTR procname);

  HANDLE _GetModuleHandle(LPCSTR modulename = NULL);
  HANDLE _GetModuleHandleW(LPCWSTR modulename = NULL);

  DWORD  _GetModuleFileName(HANDLE hmoduleornull,
                LPSTR lpbuffer,DWORD nSize);
  DWORD  _GetModuleFileNameW(HANDLE hmoduleornull,
                LPWSTR lpbuffer,DWORD nSize);

  LPSTR  _CopyOfFullExeName(HANDLE hmoduleornull = NULL);
  LPWSTR _CopyOfFullExeNameW(HANDLE hmoduleornull = NULL);

  LPSTR  _CopyOfCleanExeName(HANDLE hmoduleornull = NULL);
  LPWSTR _CopyOfCleanExeNameW(HANDLE hmoduleornull = NULL);

  LPSTR  _CopyOfExeDir(HANDLE hmoduleornull = NULL);
  LPWSTR _CopyOfExeDirW(HANDLE hmoduleornull = NULL);

  void*  _OSProcAddress(LPCSTR procname);

  void   _NoEmailsToMS(LPCSTR ApplMainModuleName);

//--------------------------------------------------------------------
#endif



