// *******************************************************************
// File EbInt64.h	     	       	      64-bit integers handling
// *******************************************************************
// Copyright @2014 by Stan Sykora and Extra Byte
#ifndef _EbInt64_h_
#define _EbInt64_h_

#include "EbTypedefs.h"

class cInt64				     // Signed 64-bit integers
// -------------------------------------------------------------------
//  Handles 64-bit integer arithmetic on 32-bit machines.
{
public:
  DWORD Low;
  LONG  High;
protected:
private:
public:
  virtual ~cInt64(){}                                    // Destructor
  					         // Constructors from:
  cInt64():Low(0),High(0){}                                 // Default
  cInt64(const cInt64& ii){Low = ii.Low;High = ii.High;}       // Copy
  cInt64(const cInt64* ii){Low = ii->Low;High = ii->High;}     // Copy
  cInt64(const DWORD l,LONG h):Low(l),High(h){}          // Components
  cInt64(const LONG  i):Low(i){High = i<0 ? -1: 0;}            // LONG
  cInt64(const DWORD i):Low(i){High = 0;}                     // DWORD
  cInt64(const REAL f);                                        // REAL
  cInt64(LPCSTR strval);                                      // ASCII
                                             // Signed conversions to:
  REAL   Double();                                             // REAL
  LPSTR  Int64ToString (LPSTR buffer=NULL,DWORD radix=10,     // ASCII
  		DWORD minwidth=0,char padding=Chr_SPACE);
                                           // Unsigned conversions to:
  inline REAL uDouble()                                        // REAL
  	       {return (4294967296.*((DWORD)High))+(DWORD)Low;}
  LPSTR  uInt64ToString(LPSTR buffer=NULL,DWORD radix=10,     // ASCII
  		DWORD minwidth=0,char padding=Chr_SPACE);

  cInt64& operator= (const cInt64& ii)          // Assignment operator
        {if (this!=&ii){Low=ii.Low;High=ii.High;} return *this;}

						    // Unary operators
  inline void Negate(){Low=~Low;High=~High;if (++Low==0) ++High;}
  inline cInt64 operator ~(){return cInt64(~Low,~High);}
  inline cInt64 operator +(){return cInt64(Low,High);}
  inline cInt64& operator ++(){if (++Low == 0) ++High; return *this;}
  inline cInt64& operator --(){if (Low-- == 0) High--; return *this;}
  inline cInt64 operator -(){Negate(); return *this;}

						  // Bitwise operators
  inline cInt64 operator &(const cInt64& arg)
    {return cInt64(Low & arg.Low,High & arg.High);}
  inline cInt64& operator &=(const cInt64& arg)
    {Low &= arg.Low; High &= arg.High; return *this;}

  inline cInt64 operator |(const cInt64& arg)
    {return cInt64(Low | arg.Low,High | arg.High);}
  inline cInt64& operator |=(const cInt64& arg)
    {Low |= arg.Low; High |= arg.High; return *this;}

  inline cInt64 operator ^(const cInt64& arg)
    {return cInt64(Low ^ arg.Low,High ^ arg.High);}
  inline cInt64& operator ^=(const cInt64& arg)
    {Low ^= arg.Low; High ^= arg.High; return *this;}

  cInt64  operator >>  (const LONG& n);
  cInt64& operator >>= (const LONG& n);
  cInt64  operator <<  (const LONG& n);
  cInt64& operator <<= (const LONG& n);

						  // Logical operators
  inline bool operator !() {return !(Low||High);}
  inline bool operator || (const cInt64& arg)
    {return (Low||arg.Low||High||arg.High);}
  inline bool operator && (const cInt64& arg)
    {return (Low||High)&&(arg.Low||arg.High);}

					       // Relational operators
  inline bool isNegative(){return High<0L;}

  inline bool operator == (const cInt64& arg)
    {return (Low==arg.Low)&&(High==arg.High);}
  inline bool operator != (const cInt64& arg)
    {return !(*this == arg);}

  inline bool operator < (const cInt64& arg)
    {return (High<arg.High)||((High==arg.High)&&(Low<arg.Low));}
  inline bool operator <= (const cInt64& arg)
    {return (High<=arg.High)||((High==arg.High)&&(Low<=arg.Low));}
  inline bool operator > (const cInt64& arg)
    {return (High>arg.High)||((High==arg.High)&&(Low>arg.Low));}
  inline bool operator >= (const cInt64& arg)
    {return (High>=arg.High)||((High==arg.High)&&(Low>=arg.Low));}

				        // Arithmetic binary operators
  cInt64  operator +  (const cInt64& arg);
  cInt64& operator += (const cInt64& arg);

  cInt64  operator -  (const cInt64& arg);
  cInt64& operator -= (const cInt64& arg);

  cInt64  operator *  (const cInt64& arg);
  cInt64& operator *= (const cInt64& arg);

  cInt64  operator /  (const cInt64& arg);
  cInt64& operator /= (const cInt64& arg);

	                   // Special, optimized arithmetic operations
  WORD  uDivideByWord(WORD w);	          // Unsigned division by WORD

private:
  LPSTR  uInt64Digits(DWORD radix,LPSTR buffer);
  LPSTR  PadAndReverse
  	   (LPSTR buffer,LPSTR endptr,DWORD minwidth,char padding);
  cInt64 Divr64u (cInt64* n,cInt64 d,cInt64 rmndr);
};

//--------------------------------------------------------------------
#endif



