// *******************************************************************
// File EbIntArr.h                  DWORD and LONG Arrays and Matrices
// *******************************************************************
// Copyright @2014 by Stan Sykora and Extra Byte.
#ifndef _EbIntArr_h_
#define _EbIntArr_h_

#include "EbInt.h"
#include "EbTemplates.h"

// ===================================================================
// Lowest-level functions which do allocation/disallocation
// ===================================================================

DWARR  NewDWARR(DWORD size);
DWARR  NewDWARRnZ(DWORD size);
inline DWARR NewCopy(DWARR arr,DWORD size)
                {return NewArrCopyT(arr,size);}
inline DWARR ResizeDWARR(DWARR* oldarr, DWORD oldsz, DWORD newsz)
                {return ResizeArr0T(oldarr,oldsz,newsz);}
inline DWARR ResizeDWARRnZ(DWARR* oldarr, DWORD oldsz, DWORD newsz)
                {return ResizeArrT(oldarr,oldsz,newsz);}
void   KillDWARR(DWARR* parr);

LARR   NewLARR(DWORD size);
LARR   NewLARRnZ(DWORD size);
inline LARR NewCopy(LARR arr,DWORD size)
                {return NewArrCopyT(arr,size);}
inline LARR ResizeLARR(LARR* oldarr, DWORD oldsz, DWORD newsz)
                {return ResizeArr0T(oldarr,oldsz,newsz);}
inline LARR ResizeLARRnZ(LARR* oldarr, DWORD oldsz, DWORD newsz)
                {return ResizeArrT(oldarr,oldsz,newsz);}
void   KillLARR(LARR* parr);

// ===================================================================
// Filling and generating grids
// ===================================================================

RETERR LArrCopy(DWORD n,LARR src,LARR dest);
RETERR LArrFill(DWORD n,LARR arr,LONG value);

RETERR DwArrCopy(DWORD n,DWARR src,DWARR dest);
RETERR DwArrFill(DWORD n,DWARR arr,DWORD value);

// ===================================================================
// LONG array info functions
// ===================================================================

inline RETERR LArrMinMax(LARR arr,DWORD n,DWORD& mnidx,DWORD& mxidx)
                {return ArrMinMaxT(arr,n,mnidx,mxidx);}
inline RETERR LArrMinMax(LARR arr,DWORD n,LONG* vmin,LONG* vmax)
                {return ArrMinMaxT(arr,n,vmin,vmax);}
inline LONG*  LArrLocateFirst(LONG* arr,DWORD n,LONG& val)
                {return IsInArrT(arr,n,val);}
inline DWORD  LArrInCount(const LONG* arr,DWORD n,LONG& val)
                {return CountInArrT(arr,n,val);}
inline BOOL   LArrIsIncreasing(const LONG* arr,DWORD n,
                 DWORD strict,DWORD* idx = NULL)
                {return IsArrIncreasingT(arr,n,strict,idx);}
inline BOOL   LArrIsDecreasing(const LONG* arr,DWORD n,
                 DWORD strict,DWORD* idx = NULL)
                {return IsArrDecreasingT(arr,n,strict,idx);}

// ===================================================================
// DWORD array info functions
// ===================================================================

inline RETERR DwArrMinMax(DWARR arr,DWORD n,DWORD& mnidx,DWORD& mxidx)
                {return ArrMinMaxT(arr,n,mnidx,mxidx);}
inline RETERR DwArrMinMax(DWARR arr,DWORD n,DWORD* vmin,DWORD* vmax)
                {return ArrMinMaxT(arr,n,vmin,vmax);}
inline DWORD* DwArrLocateFirst(DWORD* arr,DWORD n,DWORD& val)
                {return IsInArrT(arr,n,val);}
inline DWORD  DwArrInCount(const DWORD* arr,DWORD n,DWORD& val)
                {return CountInArrT(arr,n,val);}
inline BOOL   DwArrIsIncreasing(const DWORD* arr,DWORD n,
                 DWORD strict,DWORD* idx = NULL)
                {return IsArrIncreasingT(arr,n,strict,idx);}
inline BOOL   DwArrIsDecreasing(const DWORD* arr,DWORD n,
                 DWORD strict,DWORD* idx = NULL)
                {return IsArrDecreasingT(arr,n,strict,idx);}

// ===================================================================
// Operations on a two or more arrays
// ===================================================================

//--------------------------------------------------------------------
#endif



